#ifndef BSPLINE_H
#define BSPLINE_H

#include "BasisFunction.h"
#include "log4cxx/logger.h"

/**
  * A B-Spline curve controlled by some control points
  */
class BSpline
{
	static log4cxx::LoggerPtr logger;

public:
	BSpline(BasisFunction *_basisFunction, float *_controlPolygonXValues,
		float *_controlPolygonYValues, int _controlPolygonSize);
	~BSpline();

	void rebuild(float beta);
	float getSquaredLength();
	void render(float alpha);

private:
	/** Pointer to the Basis Function Set to calculate the B-Spline curve */
	BasisFunction *basisFunction;
	/** Number of control points */
	int controlPolygonSize;
	/** Number of points along the line strip to draw the B-Spline curve */
	int splinePoints;
	/** Array with X-Values of the control points with bundling strength of 1 */
	float *initialControlPolygonXValues;
	/** Array with Y-Values of the control points with bundling strength of 1 */
	float *initialControlPolygonYValues;
	/** Array with X-Values of the control points with bundling strength of beta, beta inside [0, 1] */
	float *bundlingControlPolygonXValues;
	/** Array with Y-Values of the control points with bundling strength of beta, beta inside [0, 1] */
	float *bundlingControlPolygonYValues;
	/** Array with X-Values of the spline points */
	float *xValues;
	/** Array with Y-Values of the spline points */
	float *yValues;
	/** The squared length of the B-Spline */
	float lengthSquared;

	void init();
	void computeLength();
};

#endif