#ifndef BSPLINESET_H
#define BSPLINESET_H

#include "BSpline.h"
#include "Node.h"
#include <vector>
#include <stdlib.h>

#include "BasisFunction.h"
#include "log4cxx/logger.h"

/**
  * A set of B-Spline curves
  */
class BSplineSet
{
	static log4cxx::LoggerPtr logger;

public:
	BSplineSet(int _hierarchyDepth);
	~BSplineSet();

	void addBSpline(Node **controlPolygon, int size, int lcaIndex);

	void setBundlingStrength(float _bundlingStrength);
	void setLcaRemoval(bool _removeLca);
	void useTransparency(bool _transparency);

	void renderRooted();
	void renderRadial();
	void renderBalloon();

	void sortBSplinesByLength();

private:
	/** Pointer to a vector with pointers to BSpline curves for the rooted tree without lca */
	std::vector<BSpline*> *rootedBSplineVector;
	/** Pointer to a vector with pointers to BSpline curves for the radial tree without lca */
	std::vector<BSpline*> *radialBSplineVector;
	/** Pointer to a vector with pointers to BSpline curves for the balloon tree without lca */
	std::vector<BSpline*> *balloonBSplineVector;
	/** Pointer to a vector with pointers to BSpline curves for the rooted tree with lca */
	std::vector<BSpline*> *rootedBSplineWithLcaVector;
	/** Pointer to a vector with pointers to BSpline curves for the radial tree with lca */
	std::vector<BSpline*> *radialBSplineWithLcaVector;
	/** Pointer to a vector with pointers to BSpline curves for the balloon tree with lca */
	std::vector<BSpline*> *balloonBSplineWithLcaVector;
	/** Parameter which modifies the initial control polygon, must be inside [0, 1] */
	float bundlingStrength;
	/** Number of layers of the underlying tree */
	int hierarchyDepth;
	/** Number of points used to draw the line strips representing the B-Splines */
	int pointsPerSpline;
	/** Boolean value indicating whether the least common ancestor should be removed or not */
	bool removeLca;
	/** Boolean value indicating whether transparency is used for rendering the splines or not */
	bool transparency;
	/** Array of pointers to the Basis Function Sets to calculate the B-Spline curves */
	BasisFunction **basisFunctionArray;

	void quickSort(int left, int right, std::vector<BSpline*> *splineVector);
	int split(int left, int right, std::vector<BSpline*> *splineVector);
};

#endif