#ifndef BASISFUNCTION_H
#define BASISFUNCTION_H

#include <stdlib.h>
#include <iostream>
#include "log4cxx/logger.h"

/**
  * Cox/De Boor Function Set for B-Spline generation
  */
class BasisFunction
{
	static log4cxx::LoggerPtr logger;

public:
	BasisFunction(int _numberOfControlPoints, int _splinePoints);
	~BasisFunction();

	int getNumberOfSplinePoints();
	double getValue(int i, int splinePoint);

private:
	/** Number of control points for the B-Spline */
	int numberOfControlPoints;
	/** Number of points in the line strip to draw the B-Spline */
	int splinePoints;
	/** Degree Parameter, is equal to degree - 1 */
	int p;
	/** Length of the knot vector, must be at least 2*p */
	int n;
	/** The uniform knot vector */
	int *knotVector;
	/** 2-dim Array with corresponding weights of the control points for every splinePoint */
	double **valueArray;

	void initKnotVector();
	void initValueArray();
	double coxRecursion(int i, int p, double u);
};

#endif