#ifndef CAMERA_H
#define CAMERA_H

#include <QGLWidget>
#include <QtOpenGL>
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library

#include "log4cxx/logger.h"

/**
  * Orthogonal camera for the xy-plane
  */
class Camera {
	static log4cxx::LoggerPtr logger;

	private:
		/** Camera translation in x direction */
		float translationX;
		/** Camera translation in y direction */
		float translationY;
		/** Old Camera translation in x direction */
		float oldTranslationX;
		/** Old Camera translation in y direction */
		float oldTranslationY;
		/** Zoom factor of the camera */
		float zoom;
		/** Old Zoom factor of the camera */
		float oldZoom;
		/** Determines if the right mouse button is pressed or not */
		bool rightButtonPressed;
		/** Determines if the left mouse button is pressed or not */
		bool leftButtonPressed;
		/** Width of the camera*/
		float width;
		/** Height of the camera*/
		float height;
		/** Ratio of screen in X-Direction */
		float pressXRatio;
		/** Ratio of screen in Y-Direction */
		float pressYRatio;

	public:
		Camera();
		~Camera();
		void apply(float aspectRatio);
		void processWheelEvent(QWheelEvent *event);

		void processMouseEvent(Qt::MouseButton button, bool pressed, int screenX, int screenY, int vX, int vY, int vWidth, int vHeight);
		void processMotionEvent(int screenX, int screenY, int vX, int vY, int vWidth, int vHeight);

		void reset();
		float getTranslationX();
		float getTranslationY();
		float getZoom();
		void setTranslationX(float value);
		void setTranslationY(float value);
		void setZoom(float value);
		void setCameraToBoundaries(const float *boundaries);
};

#endif