#ifndef MYGLWIDGET_H
#define MYGLWIDGET_H

#include <QGLWidget>
#include <QtOpenGL>
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library

#include "DataSet.h"
#include "Camera.h"

#include "log4cxx/logger.h"

/**
  * Widget for drawing OpenGL-related stuff
  */
 class MyGLWidget : public QGLWidget
 {	
	 static log4cxx::LoggerPtr logger;

     Q_OBJECT

	 enum TreeStyle
	 {
		 ROOTED,
		 RADIAL,
		 BALLOON
	 };

 public:
     MyGLWidget(QWidget *parent = 0);
     ~MyGLWidget();

     QSize minimumSizeHint() const;
     QSize sizeHint() const;

	 void setDataSet(DataSet* _dataSet);

 public slots:
	void setTreeStyleRooted();
	void setTreeStyleRadial();
	void setTreeStyleBalloon();

	void setBundlingStrength(int value);
	void setRelationVisability(int state);
	void setNodeVisability(int state);
	void setLineVisability(int state);
	void setCircleVisability(int state);
	void setLcaRemoval(int state);
	void useTransparency(int state);

 protected:
     void initializeGL();
     void paintGL();
     void resizeGL(int width, int height);
     void mousePressEvent(QMouseEvent *event);
     void mouseMoveEvent(QMouseEvent *event);
	 void mouseReleaseEvent(QMouseEvent *event);
	 void wheelEvent(QWheelEvent *event);

	 void renderRootedTree();
	 void renderRadialTree();
	 void renderBalloonTree();

 private:
	 /** Clear color is black */ 
 	 QColor black;
	 /** Pointer to the dataset */
	 DataSet *dataSet;
	 /** Enumeration value that indicates the tree style that is rendered */
	 TreeStyle treeStyle;
	 /** Pointer to the camera object */
	 Camera *camera;
	 /** The width of the screen */
	 int screenWidth;
	 /** The height of the screen */
	 int screenHeight;
	 /** The diameter of the nodes in the trees */
	 float nodeDiameter;
 };

 #endif