#include "FileManager.h"

#include <QMessageBox>
#include <stdio.h>
#include <iostream>
#include <string>

using namespace log4cxx;

LoggerPtr FileManager::logger(Logger::getLogger("FileManager"));

/**
  * Constructs a new FileManager object
  */
FileManager::FileManager()
{
	LOG4CXX_DEBUG(logger, "FileManager-Object created");
}

/**
  * Destroys the object
  */
FileManager::~FileManager()
{
}

/**
  * Loads the data file with the given name and returns a pointer to the generated dataset
  * @param fileName Name of the file
  * @return pointer to the generated dataset
  */
DataSet* FileManager::loadData(QString *fileName)
{
	DataSet *dataSet = NULL;

	FILE **file = new FILE*[1];
	std::string f;
	char line[40];
	std::string lineString;
	std::string value;
	size_t index;
	size_t help;

	int pcrCount = 0;
	int pcrCounter = 0;
	int relationsCount = 0;
	int relationsCounter = 0;
	int parentIndex = 0;
	int childIndex = 0;
	int originIndex = 0;
	int destinationIndex = 0;
	int hierarchyDepth = 0;

	bool hierarchyCountFound = false;
	bool hierarchyDepthFound = false;
	bool pcrCountFound = false;
	bool pcrBeginningFound = false;

	bool relationsCountFound = false;
	bool relationsBeginningFound = false;
	bool insidePCR = false;
	bool insideRelations = false;

	f = fileName->toStdString();
	fopen_s(file, f.c_str(), "rb");

	while(fgets(line, 40, *file))
	{
		lineString = std::string(line);

		if (insideRelations)
		{
			index = lineString.find_first_of("\"", 3) + 1;
			help = lineString.find_first_of("\"", index) - 1;

			value = lineString.substr(index, help - index + 1);
			originIndex = StringToInt(value);

			index = lineString.find_first_of("\"", help + 2) + 1;
			help = lineString.find_first_of("\"", index) - 1;

			value = lineString.substr(index, help - index + 1);
			destinationIndex = StringToInt(value);

			dataSet->addRelation(relationsCounter, originIndex, destinationIndex);

			relationsCounter++;

			if (relationsCounter == relationsCount)
			{
				insideRelations = false;
			}

			continue;
		}

		if (insidePCR)
		{
			index = lineString.find_first_of("\"", 5) + 1;
			help = lineString.find_first_of("\"", index) - 1;

			value = lineString.substr(index, help - index + 1);
			parentIndex = StringToInt(value);

			index = lineString.find_first_of("\"", help + 2) + 1;
			help = lineString.find_first_of("\"", index) - 1;

			value = lineString.substr(index, help - index + 1);
			childIndex = StringToInt(value);

			dataSet->getNode(parentIndex)->addChild(dataSet->getNode(childIndex));
			dataSet->getNode(childIndex)->setParent(dataSet->getNode(parentIndex));

			pcrCounter++;

			if (pcrCounter == pcrCount)
			{
				insidePCR = false;
			}
			continue;
		}
		
		if (!hierarchyCountFound) 
		{	
			index = lineString.find("\"HierarchyElements\"");
			if (index != std::string::npos) {
				help = lineString.find_last_of("\"") - 1;
				index = lineString.find_last_of("\"", help) + 1;
				value = lineString.substr(index, help - index + 1);
				
				int hierarchyElements = StringToInt(value);
				Node **nodeArray = new Node*[hierarchyElements];

				for (int i = 0; i < hierarchyElements; i++)
				{
					nodeArray[i] = new Node(i);
				}

				dataSet = new DataSet(nodeArray[0], nodeArray, hierarchyElements);
				dataSet->setHierarchyDepth(hierarchyDepth);

				hierarchyCountFound = true;
				continue;
			}
		}

		if (!hierarchyDepthFound) 
		{	
			index = lineString.find("\"HierarchyDepth\"");
			if (index != std::string::npos) {
				help = lineString.find_last_of("\"") - 1;
				index = lineString.find_last_of("\"", help) + 1;
				value = lineString.substr(index, help - index + 1);

				hierarchyDepth = StringToInt(value);
				hierarchyDepthFound = true;
				continue;
			}
		}

		if (!pcrCountFound)
		{
			index = lineString.find("\"ParentChildRelations\"");
			if (index != std::string::npos) {
				help = lineString.find_last_of("\"") - 1;
				index = lineString.find_last_of("\"", help) + 1;
				value = lineString.substr(index, help - index + 1);
				pcrCount = StringToInt(value);
				pcrCountFound = true;
				continue;
			}
		}

		if (!pcrBeginningFound)
		{
			index = lineString.find("\"PCR\"");
			if (index != std::string::npos) {
				index = lineString.find_first_of("\"", 5) + 1;
				help = lineString.find_first_of("\"", index) - 1;

				value = lineString.substr(index, help - index + 1);
				parentIndex = StringToInt(value);

				index = lineString.find_first_of("\"", help + 2) + 1;
				help = lineString.find_first_of("\"", index) - 1;

				value = lineString.substr(index, help - index + 1);
				childIndex = StringToInt(value);

				dataSet->getNode(parentIndex)->addChild(dataSet->getNode(childIndex));
				dataSet->getNode(childIndex)->setParent(dataSet->getNode(parentIndex));

				pcrCounter++;
				pcrBeginningFound = true;
				insidePCR = true;
			}
		}

		if (!relationsCountFound)
		{
			index = lineString.find("\"Relations\"");
			if (index != std::string::npos) {
				help = lineString.find_last_of("\"") - 1;
				index = lineString.find_last_of("\"", help) + 1;
				value = lineString.substr(index, help - index + 1);
				relationsCount = StringToInt(value);
				relationsCountFound = true;
				dataSet->initRelationArrays(relationsCount);
				continue;
			}
		}

		if (!relationsBeginningFound)
		{
			index = lineString.find("\"R\"");
			if (index != std::string::npos) {
				index = lineString.find_first_of("\"", 3) + 1;
				help = lineString.find_first_of("\"", index) - 1;

				value = lineString.substr(index, help - index + 1);
				originIndex = StringToInt(value);

				index = lineString.find_first_of("\"", help + 2) + 1;
				help = lineString.find_first_of("\"", index) - 1;

				value = lineString.substr(index, help - index + 1);
				destinationIndex = StringToInt(value);

				dataSet->addRelation(relationsCounter, originIndex, destinationIndex);

				relationsCounter++;
				relationsBeginningFound = true;
				insideRelations = true;
			}
		}
	}

	dataSet->init();
	
	return dataSet;
}